#import "AboutWinCtrl.h"

@implementation AboutWinCtrl

// alcuni numeri che mi servono piu' tardi
// tempo di attesa prima dell'inizio dello scrolling
#define		ABOUTWIN_STARTSCROLLDELAY	3.0F
// tempo di aggiornamento dello scrolling (in secondi)
#define		ABOUTWIN_TIMERINTERVAL		1/4
// tempo di attesa prima della ripartenza dello scrolling
#define		ABOUTWIN_INTERSCROLLDELAY	2.0F
// incremento dello scrolling
#define		ABOUTWIN_SCROLLINCREMENT	0.01F

// --------------------------------------------------------------------- 
// la solita procedura per la creazione dell'istanza condivisa
+ ( id ) 
sharedAboutWinCtrl 
{
	static	AboutWinCtrl	* locSharedAboutCtrl  = nil ;
	
	if ( ! locSharedAboutCtrl )
	{
		locSharedAboutCtrl = [[ AboutWinCtrl allocWithZone: [self zone]] init ] ;
	}
	return ( locSharedAboutCtrl );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// inizializzo la finestra caricandola dal file nib
- (id )
init 
{
	self = [ self initWithWindowNibName: @"AboutBox" ];
	return ( self );
}

// non mi preoccupo di un dealloc, perche' la finestra e' unica e creata
// una sola volta; morira' alla fine dei tempi

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// caricamento della finestra: c'e' del lavoro da fare
- (void)
windowDidLoad 
{
	NSString *creditsPath;
	NSAttributedString *creditsString;

	[ super windowDidLoad ] ;
	// inizio la posizione corrente verticale
	currentPosition = 0;
	restartAtTop = NO;
	// imposto l'istante di partenza, qualche secondo dopo l'inizio
	startTime = [NSDate timeIntervalSinceReferenceDate] + ABOUTWIN_STARTSCROLLDELAY ;
	// costuisco il path al file Credits.rtf localizzato
	creditsPath = [[NSBundle mainBundle] pathForResource:@"Credits" ofType:@"rtf"];
	// carico il contenuto del file
	creditsString = [[NSAttributedString alloc] initWithPath:creditsPath documentAttributes:nil];
	// riempio il campo di testo con il contenuto del file
	[scrollText replaceCharactersInRange:NSMakeRange( 0, 0 ) withRTF:
		[creditsString	RTFFromRange: NSMakeRange( 0, [creditsString length] ) 
				documentAttributes:nil] ];
	// calcolo l'altezza del capo di testo
	maxScrollHeight = [ creditsString size ].height ;	
	// si comincia con l'altezza a Zero
	[scrollText scrollPoint:NSMakePoint( 0, 0 )];
	// evito di associare menu al campo (font, spelling, ecc)
	[scrollText setMenu:nil ];
	// non voglio che compaia nella lista delle finestre
	[[self window] setExcludedFromWindowsMenu:YES];
	// evito di associare menu alla finestra
	[[self window] setMenu:nil];
	// porto la finestra davanti a tutti
	[[self window] makeKeyAndOrderFront:nil];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// attivita' svolte qunado la finestra si trova davanti alle altre
- (void)
windowDidBecomeKey: (NSNotification *) notification
{
	// imposto un timer: ogni x secondi verra' mandato il masseggio indicato
	// all'oggetto indicato
	scrollTimer = [NSTimer scheduledTimerWithTimeInterval: ABOUTWIN_TIMERINTERVAL
		target:self 
		selector: @selector(scrollCredits:) 
		userInfo: nil 
		repeats: YES];
	[ scrollText startSpeaking: self ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// attivita' svolte quando la finestra e' posta in secondo piano
- (void)
windowDidResignKey:(NSNotification *)notification
{
	// fermo il timer, che verosimilmente nessuno sta a guardare
	[ scrollTimer invalidate];
	[ scrollText stopSpeaking: self ];
}

// --------------------------------------------------------------------- 
// questa e' l'attivita' periodica che esegue materialmente lo scrolling
- (void)
scrollCredits: (NSTimer *)timer
{
	// vediamo se e' passato il tempo di attesa iniziale
	if ([NSDate timeIntervalSinceReferenceDate] < startTime)
		return ;
	// se arrivo qui, bisogna cominciare a scrollare
	// vedo se devo ripartire dall'inizio
	if (restartAtTop)
	{
		// e' meglio aspettare un po'
		startTime = [NSDate timeIntervalSinceReferenceDate] + ABOUTWIN_STARTSCROLLDELAY;
		// non devo piu' ricominciare dall'inizio
		restartAtTop = NO;
		// imposto la posizione dall'inizio
		[scrollText scrollPoint:NSMakePoint( 0, 0 )];            
		return;
	}
	// se arrivo qui, proseguo lo scrolling
	// vedo se per caso sono arrivato alla fine
	if (currentPosition >= maxScrollHeight) 
	{
		// sono arrivato alla fine, reimposto il timer
		startTime = [NSDate timeIntervalSinceReferenceDate] + ABOUTWIN_INTERSCROLLDELAY;
		// e riparto dall'inizio
		currentPosition = 0;
		restartAtTop = YES;
		return ;
	}
	// se sono arrivato qui, non e' successo nulla di speciale
	// imposto lo scrolling alla posizione corrente
	[scrollText scrollPoint:NSMakePoint( 0, currentPosition )];
	// che poi incremento per la volta successiva
	currentPosition += ABOUTWIN_SCROLLINCREMENT;
}

@end
