// -----------------------------------------------------------------------------
//  CCE_ElemGroup.m
//  CdCat2
//
//  Created by Livio Sandel on Mon Jan 12 2004.
//  No Copyright 2004 __MyCompanyName__. 
//
// Implementation file for a group of element

#import "CCE_ElemGroup.h"

#import "CCE_BasicForm.h"

@implementation CCE_ElemGroup

// --------------------------------------------------------------------- 
// per costruire l'elemento, inizializzo il vettore
-(id) init 
{
	self = [ super init ] ;
	if ( self )
		[ self setElemArray: [ NSMutableArray array ]];	
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// distruggo il vettore
-(void) dealloc
{
	[ elemArray release ] ;	
	[ super dealloc ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// disegno ogni singolo elemento del vettore
- (void) drawElement: (NSRect) aRect 
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	numElem = [ elemArray count ];
	for ( i = numElem-1 ; i >= 0 ; i -- )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem drawElement: aRect ];
	}
	[ self drawHandles ] ;
}

// --------------------------------------------------------------------- 
// aggiungo un elemento al vettore
- (void) addElem: (CCE_BasicForm *) elem 
{
	// inserisco davanti a tutto
	[ elemArray insertObject: elem atIndex:0];
}

// --------------------------------------------------------------------- 
// creazione di un gruppo a partire dagli elementi selezionati
- (void)
makeGroupFromSelected
{
	int		i , numElem, firstIdx = -1, lastIdx ;
	CCE_BasicForm   * elem ;
	CCE_ElemGroup   * newGrp ;
	float		xmin, xmax, ymin, ymax ;
	// ciclo su tutti gli elementi del gruppo
	lastIdx = numElem = [ elemArray count ];
	newGrp = [[[ CCE_ElemGroup alloc] init ] autorelease ];
	for ( i = numElem-1 ; i >= 0; i -- )
	{
		elem = [ elemArray objectAtIndex: i ];
		if (  [elem cceIsLocked] || ! [ elem cceIsSelected ] )
			continue ;
		// se arrivo qui, l'elemento e' libero e selezionato
		[ elem retain ];
		if ( firstIdx == -1 )
			firstIdx = i ;
		if ( lastIdx > i )
			lastIdx = i ;
		// comunque vada; lo tolgo dallo array
		[ elemArray removeObjectAtIndex: i ];
		// e lo aggiungo al nuovo gruppo
		[ [newGrp elemArray] addObject: elem ];
		[ elem release ];
	}
	// quando arrivo qui, ho costruito un nuovo gruppo
	// se non c'e' nulla, faccio nulla
	if ( firstIdx == -1)
		return ;
	// se c'e' un solo elemento, lo rimetto dov'era
	if ( [ [newGrp elemArray] count ] == 1 )
	{
		[ elemArray insertObject: [[newGrp elemArray] objectAtIndex:0] atIndex: firstIdx];
		return ;
	}
	// finalmente, se arrivo qui, ho un gruppo vero e proprio
	// aggiusto tutti i valori del gruppo
	// calcolo il rettangolo che include il tutto
	numElem = [ [newGrp elemArray] count ];
	elem = [ [newGrp elemArray] objectAtIndex: 0 ];
	ymin = [[ elem getElemTop] floatValue];
	xmin = [[ elem getElemLeft] floatValue];
	xmax = [[ elem getElemRight] floatValue];
	ymax = [[ elem getElemBottom] floatValue];
	// ciclo su tutti gli elementi del gruppo
	for ( i = 1 ; i < numElem ; i ++ )
	{
		elem = [ [newGrp elemArray] objectAtIndex: i ];
		// cerco i limiti del rettangolo circoscritto
		if ( ymin > [[ elem getElemTop] floatValue])
			ymin = [[ elem getElemTop] floatValue];
		if ( xmin > [[ elem getElemLeft] floatValue] )
			xmin = [[ elem getElemLeft] floatValue];
		if ( xmax < [[ elem getElemRight] floatValue] )
			xmax = [[ elem getElemRight] floatValue];
		if ( ymax < [[ elem getElemBottom] floatValue] )
			ymax = [[ elem getElemBottom] floatValue];
	}
	// attribuisco il rettangolo che include il gruppo
	[ newGrp setElemLimit: NSMakeRect( xmin, ymin, xmax-xmin, ymax-ymin) ];
	// tutti gli elementi interni non sono selezionati
	[ newGrp cascadeSelection: NO ];
	// il gruppo invece rimane selezionato
	[ newGrp setCceIsSelected: YES ];
	// inserisco il gruppo nel posto opportuno
	[ elemArray insertObject: newGrp atIndex: lastIdx];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// distruggo un gruppo e riporto gli elementi al livello superiore
- (void )
ungroupAndInsertIn: (CCE_ElemGroup *) masterArray  startingFrom: (int) idx
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	// per ogni elemento presento, lo aggiungo
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem setCceIsSelected: YES ] ;
		[ [masterArray elemArray] insertObject: elem atIndex: idx ];
	}
}

// --------------------------------------------------------------------- 
// spostamento degli elementi avanti ed indietro
- (void )
sendElemFrontAndBack: (int) howMuch 
{
	int		i , numElem, newIdx ;
	CCE_BasicForm   * elem ;
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	// devo distinguere tra l'avanti e l'indietro
	if ( howMuch > 0 )
	{
		for ( i = 1 ; i < numElem ; i ++ )
		{
			elem = [ elemArray objectAtIndex: i ];
			if ( [elem cceIsLocked] || ! [ elem cceIsSelected ] )
				continue ;
			// se arrivo qui, devo spostare l'elemento
			[ elem retain ];
			// comunque vada; lo tolgo dallo array
			[ elemArray removeObjectAtIndex: i ];
			// vedo cosa ne devo fare
			if ( howMuch == +1 )
					newIdx = i-1 ;
			else	newIdx = 0 ;
			[ elemArray insertObject: elem atIndex: newIdx ];
			[ elem release ];
		}
	}
	else
	{
		for ( i = numElem-2 ; i >= 0 ; i -- )
		{
			elem = [ elemArray objectAtIndex: i ];
			if ( [elem cceIsLocked] || ! [ elem cceIsSelected ] )
				continue ;
			// se arrivo qui, devo spostare l'elemento
			[ elem retain ];
			// comunque vada; lo tolgo dallo array
			[ elemArray removeObjectAtIndex: i ];
			// vedo cosa ne devo fare
			if ( howMuch == -1 )
					newIdx = i+1 ;
			else	newIdx = numElem-1 ;
			[ elemArray insertObject: elem atIndex: newIdx ];
			[ elem release ];
		}
	}
}

// --------------------------------------------------------------------- 
// attribuzione a cascata dell'attributo di selezione
- (void)
cascadeSelection:(BOOL)newIsSelected
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		if ( [elem isKindOfClass:[CCE_ElemGroup class]] )
			[ ((CCE_ElemGroup*) elem) cascadeSelection: newIsSelected ];
		[ elem setCceIsSelected: newIsSelected ];
	}
}

// --------------------------------------------------------------------- 
// 
- (void) 
selectElemsInRect: (NSRect) selRect addingToSelez: (BOOL) add2sel
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem selectMeIfInRect: selRect addingToSelez: add2sel] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
moveSelectedElements: (NSPoint) distance
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem moveMeIfSelectBy: distance ];
	}
}

// --------------------------------------------------------------------- 
- (NSMutableArray *) 
getSelectedGraphics
{
	NSMutableArray  * selArr = [[ NSMutableArray alloc] init ] ;
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		if ([ elem cceIsSelected] )
			[ selArr addObject: elem ];
	}	
	return ( selArr );
}

// --------------------------------------------------------------------- 
- (void)
moveMeIfSelectBy: (NSPoint) distance
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	if ( (! cceIsLocked) && cceIsSelected )
	{
		[ super moveMeIfSelectBy: distance ];
		// ciclo su tutti gli elementi del gruppo
		numElem = [ elemArray count ];
		for ( i = 0 ; i < numElem ; i ++ )
		{
			elem = [ elemArray objectAtIndex: i ];
			[ elem moveMeBy: distance ];
		}
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
moveMeBy: (NSPoint) distance
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	[ super moveMeBy: distance ];
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem moveMeBy: distance ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
setCceIsLocked:(BOOL)newVal
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	//assegno il valore a me stesso
	[ super setCceIsLocked: newVal ];
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem setCceIsLocked: newVal ];
	}
}

// --------------------------------------------------------------------- 
// accessor
- (NSMutableArray *)elemArray {
    return elemArray ;
}

- (void)setElemArray:(NSMutableArray *)newElemArray {
    if (elemArray != newElemArray) {
        [elemArray release];
        elemArray = [newElemArray retain];
    }
}

// --------------------------------------------------------------------- 
- (void)
printMeWithLevel: (int) lev
{
	int		i , numElem ;
	char	blankChar[40];
	CCE_BasicForm   * elem ;
	NSString	* msg ;
	if ( lev > 10 ) lev = 10 ;
	for ( i = 0; i < lev*4 ; i ++ )
		blankChar[i]= ' ';
	blankChar[lev*4] = 0 ;
	msg = [ NSString stringWithCString: blankChar];
	NSLog( @"%@ -> ElemGroup of %d elements (R: %7.2f %7.2f S: %7.2f %7.2f) Sel: %2d, Lk: %2d", 
		msg, [elemArray count], 
		elemLimit.origin.x, elemLimit.origin.y,
		elemLimit.size.width, elemLimit.size.height, 
		cceIsSelected, cceIsLocked );

	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem printMeWithLevel: (lev+1) ];
	}
}

@end
