// -----------------------------------------------------------------------------
//  CCE_Line.m
//  CdCat2
//
//  Created by Livio Sandel on Mon Jan 12 2004.
//  No Copyright 2004 __MyCompanyName__. 
//
// Implementation file for drawing a simple line

#import "CCE_Line.h"

#include <math.h>
#import "CoverView.h"

@implementation CCE_Line

// --------------------------------------------------------------------- 
// inizializzione: ho bisogno di due punti
-(id) initStartPt: (NSPoint) stPt toPt: (NSPoint) endPt 
{
	self = [ super init ];
	if ( self ) 
	{
		numOfHdl = 2 ;
		[ self buildMeWithStartPt: stPt andPt: endPt ];
		[ self setCceIsStroked: TRUE ];
	}
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void) buildHdlList 
{
		// gli handle sono genericamente ai quattro estremi del
		// rettangolo limite e nei quattro punti mediani
		hdlList[0] = startPoint ;
		hdlList[1] = endPoint ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
buildMeWithStartPt: (NSPoint) stPt andPt: (NSPoint) endPt 
{
		// per mia praticita', costruisco un percorso
		NSBezierPath *path = [NSBezierPath bezierPath];
		// assegno i punti di partenza ed arrivo
		startPoint = stPt ;
		endPoint = endPt ;
		numOfHdl = 2 ;
		// costruisco il percorso
        [path moveToPoint: startPoint ];
        [path lineToPoint: endPoint ];
		// lo assegno
		[ self setTheLine: path ];
		// metto a posto il rettangolo che lo contiene
		[self setElemLimit: [theLine bounds] ];
		// variabile di appoggio
		delta = 1.0F / sqrt( ( endPoint.x - startPoint.x)*( endPoint.x - startPoint.x) +
				( endPoint.y - startPoint.y)*( endPoint.y - startPoint.y) );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// distruggo tutto
-(void) dealloc
{
	[ theLine release ] ;	
	[ super dealloc ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// disegno la linea
- (void) drawElement: (NSRect) inRect 
{
	// imposto il colore
	[ cceLineColor set ];
	// imposto lo spessore della linea
	[ theLine setLineWidth: [ self cceLineWidth ]];
	// disengo la linea
	[ theLine stroke ];
	[ self drawHandles ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// per capire se qualcuno ha fatto clic su di me 
- (BOOL) 
checkClickOnMe:(NSPoint)point 
{
	float   ev ;
	NSRect locRect ;
	// rettangolo in cui si trova la linea
	locRect = NSInsetRect( [self elemLimit], -2, -2 );
	// se sono dentro, proseguo
	if (NSPointInRect(point, locRect ) )
	{
		// retta per un punto, e distanza di un punto da una retta
		ev = ( endPoint.x - startPoint.x)* (point.y - startPoint.y) -
			( endPoint.y - startPoint.y)* (point.x - startPoint.x) ;
		if ( ev < 0 ) ev = -ev ;
		// se la distanza e' inferiore allo spessore della linea piu'
		// una certa tolleranza per essere sicuri di beccare il punto...
		if ( ev * delta < ([ self cceLineWidth] + 3) )
			return ( TRUE ) ;
	}
	// se arrivo qui, mancato!
	return ( FALSE );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (BOOL)
handleCreation: (NSEvent *)theEvent 
	inView:		(CoverView *)view 
{
	// recupero la coordinata locale
    NSPoint point1 = [view convertPoint:[theEvent locationInWindow] fromView:nil];
	NSPoint point ;
    BOOL	shiftKeyPress ;
	// inizializzo l'aspetto dell'oggetto (praticamente vuoto)
	[ self buildMeWithStartPt: point1 andPt: point1 ] ;
	// continuo a ciclare in attesa di un mouseup
	do  {
		// recupero il prossimo evento, che sia un movimento del mouse 
		// oppure un mouseup (che finisce il tutto)
        theEvent = [[view window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];  
		shiftKeyPress = (([theEvent modifierFlags] & NSShiftKeyMask) ? YES : NO);
		// recupero la coordinata locale
		point = [view convertPoint:[theEvent locationInWindow] fromView:nil];
		if ( shiftKeyPress ) 
		{				
			// costringo il rettangolo ad essere quadrato
			float   delta1 = ( point.x - point1.x ) ;
			float   delta2 = ( point.y - point1.y ) ;
			float	dirx = 1 ;
			float	diry = 1 ;
			
			if ( delta1 < 0 ) 
			{
				delta1 = - delta1 ; dirx = -1 ;
			}
			if ( delta2 < 0 ) 
			{
				delta2 = - delta2 ; diry = -1 ;
			}
			// piglio la dimensione piu' corta
			if ( delta1 < delta2*0.2 )
			{
				point.x = point1.x ;
			}
			else if ( delta2 < delta1*0.2 )
			{
				point.y = point1.y  ;
			}
			else if ( delta1 < delta2 ) 
			{
				point.y = point1.y + diry * delta1 ;
			}
			else
			{
				point.x = point1.x + dirx * delta2 ;
			}
		}
		// aggiusto l'aspetto dell'oggetto
		[ self buildMeWithStartPt: point1 andPt: point ] ;
		// forzo un ridisegno della finestra
		[ view setNeedsDisplay: YES ];
    }
	while ( [theEvent type] != NSLeftMouseUp) ;
	// arrivato qui, ho finito il tutto
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (BOOL)
handleResize: (NSEvent *)theEvent 
	inView:		(CoverView *)view 
	movingHandle: (short) hdlIdx 
{	
	NSPoint fixedPoint = ( hdlIdx == 0) ? hdlList[ 1 ] : hdlList[ 0 ];
	NSPoint point ;
	// continuo a ciclare in attesa di un mouseup
	do  {
		// recupero il prossimo evento, che sia un movimento del mouse 
		// oppure un mouseup (che finisce il tutto)
        theEvent = [[view window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];  
		// recupero la coordinata locale
		point = [view convertPoint:[theEvent locationInWindow] fromView:nil];
		// aggiusto l'aspetto dell'oggetto
		[ self buildMeWithStartPt: fixedPoint andPt: point ] ; 
		// forzo un ridisegno della finestra
		[ view setNeedsDisplay: YES ];
    }
	while ( [theEvent type] != NSLeftMouseUp) ;
	// arrivato qui, ho finito il tutto
	return YES;
}

// --------------------------------------------------------------------- 
- (void) 
selectMeIfInRect: (NSRect) selRect addingToSelez: (BOOL) add2sel
{
	BOOL	inrect = NSPointInRect(startPoint, selRect) && NSPointInRect(endPoint, selRect) ;
	// se non devo aggiungere alla selezione
	if ( ! add2sel )
	{
		// vince il criterio del rettangolo
		[ self setCceIsSelected: inrect ];
	}
	else
	{
		// qui e' da aggiungere alla selezione
		// se sono dentro il rettangolo
		if ( inrect )
			// per certo lo seleziono
			[ self setCceIsSelected: YES ];
		// altrimenti lascio la selezione precedente
	}
}

// --------------------------------------------------------------------- 
- (void)
moveMeIfSelectBy: (NSPoint) distance
{
	if ( (! cceIsLocked) && cceIsSelected )
	{
		NSPoint p1 = NSMakePoint( startPoint.x + distance.x,  startPoint.y + distance.y);
		NSPoint p2 = NSMakePoint( endPoint.x + distance.x,  endPoint.y + distance.y);
		[ self buildMeWithStartPt: p1 andPt: p2 ];
	}
}

// --------------------------------------------------------------------- 
// accessor
- (NSBezierPath *)theLine {
    return theLine ;
}

- (void)setTheLine:(NSBezierPath *)newTheLine {
    if (theLine != newTheLine) {
        [theLine release];
        theLine = [newTheLine retain];
    }
}

// --------------------------------------------------------------------- 
- (void)
printMeWithLevel: (int) lev
{
	int i;
	char	blankChar[40];
	NSString	* msg ;
	if ( lev > 10 ) lev = 10 ;
	for ( i = 0; i < lev*4 ; i ++ )
		blankChar[i]= ' ';
	blankChar[lev*4] = 0 ;
	msg = [ NSString stringWithCString: blankChar];
	NSLog( @"%@ -> Line (P: %7.2f %7.2f P: %7.2f %7.2f) Lw: %6.2f, F: %2d, S: %2d, Sel: %2d, Lk: %2d", msg, 
		startPoint.x, startPoint.y,
		endPoint.x , endPoint.y, 
		cceLineWidth, cceIsFilled,cceIsStroked, cceIsSelected, cceIsLocked );
}


@end
