// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  CatFileInfo.m
//  Created by djzero00 - - Copyright (c) 2002
//
// realizzazione della classe che modella le informazioni relative ad un file

#import		"CatFileInfo.h"

#import		"djZeroUtils.h"
#import		"WaitPanCtrl.h"

@implementation CatFileInfo

// --------------------------------------------------------------------- 
// inizializzazione dell'istanza in base al nome completo
- (id) 
initWithPath: (NSString*) aFile
{
	NSImage			* tmpImg ;
	// piglio l'oggetto FileManager di default
	NSFileManager	*manager = [NSFileManager defaultManager];
	// ed in base a questo recupero le informazioni proprie del file
	NSDictionary	*fattrs	 = [manager fileAttributesAtPath: aFile traverseLink:NO];

	// inizializzo la classe madre
	[ super init ];
	// ed ora passo ad inizializzare tutte le variabili d'istanza
	// path completo e nome sono facili
	[ self setFileFullPath: aFile ];
	// qui uso un metodo standard per recuperare il nome dal path completo
	[ self setFileName: [aFile lastPathComponent]];
	// con i metodi di comodo della classe NSDictionary recupero tutte le 
	// info codificate
	[ self setModDate: [fattrs fileModificationDate] ];
	[ self setCreatDate: [fattrs fileCreationDate] ];
	[ self setFileSize: [ fattrs fileSize ] ];
	[ self setOwnGroupName: [ fattrs fileGroupOwnerAccountName ] ];
	[ self setOwnerName:[ fattrs fileOwnerAccountName ] ];
	[ self setFilePosixPerm: [ fattrs filePosixPermissions ] ] ;
	[ self setFileType: [ fattrs fileType ] ] ;
	[ self setCreatorCode: [ fattrs fileHFSCreatorCode] ];
	[ self setTypeCode: [ fattrs fileHFSTypeCode] ];
	[ self setIs2Print: TRUE ];
	// se il file e' effettivamente un vero file
	if ( [[self fileType] isEqualToString: NSFileTypeRegular ])
	{
		OSStatus 		status ;
		FSRef 			myFSRef;
		FSCatalogInfo		catinfo ;
		unsigned long long	tmpSize ;
		// potrebbe esserci una resource fork,  costruisco una FSRef
		status = FSPathMakeRef ([aFile fileSystemRepresentation], &myFSRef,  NULL); 
		// e poi indago con Carbon le dimensioni della resource
		if (status == noErr)
			status = FSGetCatalogInfo (&myFSRef, kFSCatInfoRsrcSizes, & catinfo, NULL, NULL, NULL);
		//aggiusto le dimensioni del file
		tmpSize = [self fileSize] + catinfo.rsrcLogicalSize ;
		// e re-imposto tale valore
		[ self setFileSize: tmpSize ];
	}
	// prelevo ed assegno l'icona del file, ridotta ai minimi termini
	tmpImg = reduceImageToIcon( [ [ NSWorkspace sharedWorkspace] iconForFile: aFile ] ); ;
	[ self setFileIcon: tmpImg ];
	// aggiorno la barra di attesa
	[ WaitPanCtrl setCurrentValue: ([ WaitPanCtrl getCurrentValue ] + 1)] ;
	return ( self ) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// dealloco tutta la memoria che ho dovuto costruire 
- (void) 
dealloc 
{
	[ fileIcon release ];
	[ fileFullPath release ];
	[ fileName release ] ;
	[ modDate release ] ;
	[ creatDate release ] ;
	[ ownGroupName release ] ;
	[ ownerName release ];
	[ fileType release ] ;	
	[ super dealloc ] ;
}

// --------------------------------------------------------------------- 
// metodo per il salvataggio/ripristino dell'istanza da file
- (void)
encodeWithCoder:(NSCoder *)encoder 
{
	// non devo chiamare super perche' sono diretto discendente di NSObject
	// gli oggetti li scrivo col metodo appropriato
	[ encoder encodeObject: [ self fileFullPath ] ] ;
	[ encoder encodeObject: [ self fileName ] ] ;
	[ encoder encodeObject: [ self modDate ] ] ;
	[ encoder encodeObject: [ self creatDate ] ] ;
	[ encoder encodeObject: [ self ownGroupName ] ] ;
	[ encoder encodeObject: [ self ownerName ] ] ;
	[ encoder encodeObject: [ self fileType ] ] ;
	[ encoder encodeObject: [ self fileIcon ] ] ;
	// tutti gli altri con i metodi specifici dei tipi C
	[ encoder encodeValueOfObjCType: @encode(long long) at: & fileSize];
	[ encoder encodeValueOfObjCType: @encode(unsigned long) at: & filePosixPerm];
	[ encoder encodeValueOfObjCType: @encode(OSType) at: & creatorCode];
	[ encoder encodeValueOfObjCType: @encode(OSType) at: & typeCode];
	[ encoder encodeValueOfObjCType: @encode(BOOL) at: & is2Print ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo per il salvataggio/ripristino dell'istanza da file
- (id)initWithCoder:(NSCoder *)decoder 
{
	// e' fondamentale l'ordine, che deve essere lo stesso di encodeWithCoder
	// non devo chiamare super perche' sono diretto discendente di NSObject
	[ self setFileFullPath: [ decoder decodeObject ] ];
	[ self setFileName: [ decoder decodeObject ] ];
	[ self setModDate: [ decoder decodeObject ] ];
	[ self setCreatDate: [ decoder decodeObject ] ];
	[ self setOwnGroupName: [ decoder decodeObject ] ];
	[ self setOwnerName: [ decoder decodeObject ] ];
	[ self setFileType: [ decoder decodeObject ] ];
	[ self setFileIcon: [ decoder decodeObject ] ];
	[ decoder decodeValueOfObjCType: @encode(long long) at: & fileSize];
	[ decoder decodeValueOfObjCType: @encode(unsigned long) at: & filePosixPerm];
	[ decoder decodeValueOfObjCType: @encode(OSType) at: & creatorCode];
	[ decoder decodeValueOfObjCType: @encode(OSType) at: & typeCode];
	[ decoder decodeValueOfObjCType: @encode(BOOL) at: & is2Print];
	return self ;
}

// --------------------------------------------------------------------- 
// metodi accessor per tutte le variabili d'istanza
- (NSString *)fileName {
    return [[fileName retain] autorelease];
}

- (void)setFileName:(NSString *)newFileName {
    if (fileName != newFileName) {
        [fileName release];
        fileName = [newFileName copy];
    }
}

- (NSDate *)modDate {
    return [[modDate retain] autorelease];
}

- (void)setModDate:(NSDate *)newModDate {
    if (modDate != newModDate) {
        [modDate release];
        modDate = [newModDate copy];
    }
}

- (NSDate *)creatDate {
    return [[creatDate retain] autorelease];
}

- (void)setCreatDate:(NSDate *)newCreatDate {
    if (creatDate != newCreatDate) {
        [creatDate release];
        creatDate = [newCreatDate copy];
    }
}

- (NSString *)ownGroupName {
    return [[ownGroupName retain] autorelease];
}

- (void)setOwnGroupName:(NSString *)newOwnGroupName {
    if (ownGroupName != newOwnGroupName) {
        [ownGroupName release];
        ownGroupName = [newOwnGroupName copy];
    }
}

- (NSString *)ownerName {
    return [[ownerName retain] autorelease];
}

- (void)setOwnerName:(NSString *)newOwnerName {
    if (ownerName != newOwnerName) {
        [ownerName release];
        ownerName = [newOwnerName copy];
    }
}

- (OSType)creatorCode {
    return creatorCode ;
}

- (void)setCreatorCode:(OSType)newCreatorCode {
    creatorCode = newCreatorCode ;
}

- (OSType)typeCode {
    return typeCode;
}

- (void)setTypeCode:(OSType)newTypeCode {
   typeCode = newTypeCode;
}

- (BOOL)is2Print {
    return is2Print ;
}

- (void)setIs2Print:(BOOL)newIs2Print {
        is2Print = newIs2Print;
}

- (NSImage *)fileIcon {
    return [[fileIcon retain] autorelease];
}

- (void)setFileIcon:(NSImage *)newFileIcon {
    if (fileIcon != newFileIcon) {
        [fileIcon release];
        fileIcon = [newFileIcon retain];
    }
}

- (NSString *)fileFullPath {
    return [[fileFullPath retain] autorelease];
}

- (void)setFileFullPath:(NSString *)newFileFullPath {
    if (fileFullPath != newFileFullPath) {
        [fileFullPath release];
        fileFullPath = [newFileFullPath copy];
    }
}

- (NSString *)fileType {
    return [[fileType retain] autorelease];
}

- (void)setFileType:(NSString *)newFileType {
    if (fileType != newFileType) {
        [fileType release];
        fileType = [newFileType copy];
    }
}

- (unsigned long long)fileSize {
    return fileSize;
}

- (void)setFileSize:(unsigned long long)newFileSize {
        fileSize = newFileSize;
}

- (unsigned long )filePosixPerm {
    return filePosixPerm;
}

- (void)setFilePosixPerm:(unsigned long )newFilePosixPerm {
        filePosixPerm = newFilePosixPerm;
}

@end
