#import "ListWinCtl.h"

#import		"ChooseVolCtrl.h"
#import		"ImageAndTextCell.h"
#import		"CatFileInfo.h"
#import		"UserPrefs.h"
#import		"CatDataSrc.h" ;
#import		"CdCatDoc.h" ;
#import		"LocalFormatters.h"

void	configureColumn( NSTableColumn *outView, NSString * colId ) ;
void	setupColumn( NSOutlineView *outView, BOOL addRem, NSString * colId ) ;

@implementation ListWinCtl

// --------------------------------------------------------------------- 
// predisposizione del contenuto della finestra
- (void) 
windowDidLoad
{
	ImageAndTextCell	* imageAndTextCell = nil;
	NSTableColumn		* tableColumn = nil;
	CatDataSrc			* myDataSrc ;
	NSButtonCell		* add2print = nil ;
	
	[super windowDidLoad ];
	// associo questa finestra alla sorgente dati
	myDataSrc = [ [self document] dataSource] ;
	// inserisco una cella custom nella outlineView; la cella standard gestisce solo
	// testo, questa nuova testo ed una immagine
	tableColumn = [fullList tableColumnWithIdentifier: COLID_FILENAME];
	// creo l'oggetto autorelease perche' passa in carico alla tableColumn
	imageAndTextCell = [[[ImageAndTextCell alloc] init] autorelease];
	[tableColumn setDataCell:imageAndTextCell];
	// aggiungo anche la colonna is2Print
	tableColumn = [fullList tableColumnWithIdentifier: COLID_ADD2PRINT];
	add2print = [[[ NSButtonCell alloc] init ] autorelease ];
	[ add2print setButtonType: NSSwitchButton ];
	[ add2print setTitle:@""] ;
	[ tableColumn setWidth: 16 ];
	[ tableColumn setResizable: FALSE ];
	[ [ tableColumn headerCell] setStringValue: @"P?" ];
	[ tableColumn setDataCell:add2print];

	// assegno alla colonna presente l'immagine che indica l'ordinamento
	[ fullList setIndicatorImage: [NSImage imageNamed: @"imgUpS"] inTableColumn: tableColumn ];
	[ fullList setDelegate:   myDataSrc ];
	[ fullList setDataSource: myDataSrc ];
	// salvo nelle prefs le dimensioni delle colonne della outlineView
	[ fullList setAutosaveName: @"FullListViewOptions" ];
	[ fullList setAutosaveTableColumns: YES ];
	// facendo doppio clic, si cambia la direzione di ordinamento
	[ fullList setTarget: self ];
	[ fullList setDoubleAction: @selector( changeSortOrder:) ];

	// registro la finestra che accetti drag
	[ [ self window] registerForDraggedTypes: 
		[NSArray arrayWithObject: NSFilenamesPboardType]  ];

	// attacco la toolbar alla finestra
	[self addToolbarToWin: [ self window] ];	

	// mi abbono all'evento: pref aggiornate
	[[ NSNotificationCenter defaultCenter] addObserver: self
		selector: @selector( prefsUpdated: )
		name: PREF_UPDATE_NOTIFICATION object: nil ] ;
	// recupero le pref ed adeguo la finestra
	// in realta', faccio finta che siano state aggiornate le prefs
	[[NSNotificationCenter defaultCenter]
		postNotificationName: PREF_UPDATE_NOTIFICATION object:self];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// il titolo della finestra e' particolarizzato
- (NSString *)
windowTitleForDocumentDisplayName:(NSString *)displayName
{
	return ( [ @"Catalog List: " stringByAppendingString: displayName] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// con questo metodo in qualche modo dico che questo controllore di finestr
// e' il controllore principale
- (BOOL)
shouldCloseDocument
{
	return ( YES );
}

// --------------------------------------------------------------------- 
// metodi per aggiungere / rimuovere elementi dal catalogo
// aggiunta di un elemento al catalogo
- (void) 
addFileItem: (id)sender
{
	ChooseVolCtrl *tmpCVCtrl = [[ ChooseVolCtrl alloc] init ] ;
	// comincio la procedura: mi metto modale per il documento
	// alla chisura della sheet, sara' invocato il metodo addVolume:...
	[ NSApp beginSheet: [ tmpCVCtrl window ]
		modalForWindow: [ NSApp mainWindow ]  
		modalDelegate: [ self document]
		didEndSelector: @selector( addVolume2Cat:returnCode:contextInfo: )  
		contextInfo: nil
	];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// rimozione di un elemento dal catalogo
- (void) 
delItem: (id)sender
{
	FileStruct * fInfo ;
	// guardo se c'e' una riga selezionata
	int	rowsel = [ fullList selectedRow ];
	if ( rowsel == -1 ) 
	{
		// no, lascio stare
		NSBeep();	// bip!!
		return ;
	}
	// se arrivo qui, c'e' una riga selezionata
	fInfo = [ fullList itemAtRow: rowsel ] ;
	// adesso voglio eliminare questo elemento da dataSource
	[ [[ self document] dataSource] removeEntry: fInfo ];
}

// --------------------------------------------------------------------- 
// metodo invocato da un doppio clic su di un elemento della outlineView
- (void)
changeSortOrder: (id) sender 
{
	NSTableColumn	* tableColumn ;
	CatDataSrc		* myDataSrc ;

	// associo questa finestra alla sorgente dati
	myDataSrc = [ [self document] dataSource] ;
	// voglio che sia selezionata una intera colonna
	if ( [ sender selectedColumn ] == -1 ) return ;
	// determino allora quale colonna e' selezionata
	tableColumn = [ [fullList tableColumns] objectAtIndex: [ fullList clickedColumn ]];
	if ([ [ tableColumn identifier] isEqualToString: COLID_ADD2PRINT]) return ;
	// se la colonna di ordinamento non cambia
	if ([ [ tableColumn identifier] isEqual: [ myDataSrc orderColumn ]])
	{
		// rovescio l'ordine e cambio l'immagine (lo faccio poi)
		[ myDataSrc setOrderDirection: ! [ myDataSrc orderDirection ] ];
	}
	else
	{
		[ fullList setIndicatorImage: nil 
			inTableColumn: [ fullList 
				tableColumnWithIdentifier: [ myDataSrc orderColumn ] 
			] 
		];
		// rovescio l'ordine negando il valore corrente
		[ myDataSrc setOrderColumn: [ tableColumn identifier]];
	}
	// in ogni caso imposto l'immagine nello header della colonna
	if ( [ myDataSrc orderDirection ])
			[ fullList setIndicatorImage: [NSImage imageNamed: @"imgUpS"] inTableColumn: tableColumn ];
	else	[ fullList setIndicatorImage: [NSImage imageNamed: @"imgDwS"] inTableColumn: tableColumn ];
	// rinfresco la finestra con i nuovi dati
	[ fullList reloadData ];
}

// ----------------------------------------------------------------------------------------
// ricostruzione della outlineView in seguito ad una modifica delle preferenze
- (void)
prefsUpdated: ( NSNotification *) notification
{
	int fi, ti ;
	// recupero le preferences e mostro o meno la colonna
	setupColumn( fullList, [[ UserPrefs getPrefValue: keyColModDate] boolValue], COLID_MODDATE );
	setupColumn( fullList, [[ UserPrefs getPrefValue: keyColCreatDate] boolValue], COLID_CREATDATE );
	setupColumn( fullList, [[ UserPrefs getPrefValue: keyColFileSize] boolValue], COLID_FILESIZE );
	setupColumn( fullList, [[ UserPrefs getPrefValue: keyColGroupName] boolValue], COLID_GROUPNAME );
	setupColumn( fullList, [[ UserPrefs getPrefValue: keyColOwnerName] boolValue], COLID_OWNERNAME );
	setupColumn( fullList, [[ UserPrefs getPrefValue: keyColPosixPerm] boolValue], COLID_POSIXPERM );
	setupColumn( fullList, [[ UserPrefs getPrefValue: keyColCreator] boolValue], COLID_OSCREATOR );
	setupColumn( fullList, [[ UserPrefs getPrefValue: keyColOSType] boolValue], COLID_OSTYPE );
	// questa colonna e' sempre visualizzata
	setupColumn( fullList, TRUE , COLID_ADD2PRINT );
	// sposto la colonna is2Print in ultima posizione
	fi = [ fullList columnWithIdentifier: COLID_ADD2PRINT ];
	ti = [ fullList numberOfColumns] - 1 ;
	[ fullList moveColumn: fi toColumn: ti ];
	[ fullList reloadData ];
}

// ----------------------------------------------------------------------------------------
// metodi per la costruzione e gestione della toolbar

#define		CatDocToolbarId		@"tbName_CatDoc"
#define		TBItId_AddItem		@"tbItem_AddItem"
#define		TBItId_DelItem		@"tbItem_DelItem"
#define		TBItId_SaveItem		@"tbItem_SaveItem"

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo per la costruzione della toolbar
- (void) 
addToolbarToWin: (NSWindow *) currWin
{
	// costruisco la nuova istanza della toolbar
	// la faccio autorelease perche' sara' poi ritenuta dalla finestra
	NSToolbar *toolbar = [[[NSToolbar alloc] initWithIdentifier: CatDocToolbarId] autorelease];

	// imposto le proprieta' di base della toolbar
	// permetto che l'utente la modifichi
	[toolbar setAllowsUserCustomization: YES];
	// lascio che conservi lo stato tra successivi lanci
	[toolbar setAutosavesConfiguration: YES];
	// dico che mostri solo le icone
	[toolbar setDisplayMode: NSToolbarDisplayModeIconOnly];
	// dico che il delegato della toolbar e' il winctl stesso
	[toolbar setDelegate: self];
	// attacco la toolbar alla finestra
	[ currWin setToolbar: toolbar ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo delegato obbligatorio
// dato un identificatore di item, restuisce l'item vero e proprio
	// questo metodo serve per dare corpo agli item non standard
- (NSToolbarItem *) 
toolbar: (NSToolbar *)toolbar 
	itemForItemIdentifier: (NSString *) itemIdent 
	willBeInsertedIntoToolbar:(BOOL) willBeInserted 
{
	// costruisco una istanza di item della toolbar
	// come al solito, e' autorelase, verra' ritenuto dalla toolbar
	NSToolbarItem *toolbarItem = [[[NSToolbarItem alloc] initWithItemIdentifier: itemIdent] autorelease];
	
	// item della toolbar per aggiungere un volume
	if ([itemIdent isEqual: TBItId_AddItem]) 
	{
		// do due nomi all'icona di aggiunta volume: nome per la toolbar
		[toolbarItem setLabel: @"Add"];
		// nome per il dialogo di personalizzazione
		[toolbarItem setPaletteLabel: @"Add Volume"]; 
		// predispongo la scritta di aiuto (il tooltip)
		[toolbarItem setToolTip: @"Add a Volume to the Catalog"];
		// predispongo l'icona di questo item
		[toolbarItem setImage: [NSImage imageNamed: @"Add"]];
		// quando l'item e' cliccato, deve mandare al documento (self)... 
		[toolbarItem setTarget: self];
		// ...il messaggio opportuno
		[toolbarItem setAction: @selector(addFileItem:)];
		return toolbarItem;
	}
	// ripeto il giochetto per la cancellazione di un elemento di catalogo
	if ([itemIdent isEqual: TBItId_DelItem]) 
	{
		[toolbarItem setLabel: @"Delete"];
		[toolbarItem setPaletteLabel: @"Delete Item"];
		[toolbarItem setToolTip: @"Remove an Item from the Catalog"];
		[toolbarItem setImage: [NSImage imageNamed: @"del"]];
		[toolbarItem setTarget: self];
		[toolbarItem setAction: @selector(delItem:)];
		return toolbarItem;
	}
	// e per il salvataggio del catalogo
	if ([itemIdent isEqual: TBItId_SaveItem]) 
	{
		[toolbarItem setLabel: @"Save"];
		[toolbarItem setPaletteLabel: @"Save Item"];
		[toolbarItem setToolTip: @"Save the Catalog"];
		[toolbarItem setImage: [NSImage imageNamed: @"Save"]];
		[toolbarItem setTarget: [self document] ];
		// chiamo un metodo di default
		[toolbarItem setAction: @selector(saveDocument:)];
		return toolbarItem;
	}
	// gli altri dovrebbero essere standard
	return ( nil );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo delegato obbligatorio
// questo metodo dice quali sono gli item di default della toolbar
	// sono gli item presenti alla creazione e quando l'utente ripristina
- (NSArray *) 
toolbarDefaultItemIdentifiers: (NSToolbar *) toolbar 
{
	// la lista e' l'elenco ordinato degli identificatori degli elementi
	return [NSArray arrayWithObjects:	
		TBItId_AddItem, 			// aggiunta elemento
		TBItId_SaveItem, 			// salvataggio catalogo
		NSToolbarSeparatorItemIdentifier, 	// separatore
		TBItId_DelItem,				// cancellazione
		NSToolbarSeparatorItemIdentifier, 	// separatore
		NSToolbarPrintItemIdentifier, 		// stampa catalogo
		NSToolbarFlexibleSpaceItemIdentifier, 	// spazio flessibile
		NSToolbarCustomizeToolbarItemIdentifier,// personalizzazione
		nil];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo delegato obbligatorio
// questo metodo dice quali sono gli item possibili della toolbar
	// sono TUTTI gli item che possono entrare nella toolbar, e quindi
	// mostrati nel dialogo di personalizzazione
- (NSArray *) 
toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar 
{
	// normalmente, nessun elemento e' previsto presente, quindi bisogna
	// inserire tutti gli elementi che si pensa possano essere presenti;
	// in particolare, separatori e spazi vari...
	return [NSArray arrayWithObjects: 
		TBItId_AddItem, TBItId_SaveItem, TBItId_DelItem,	// i miei tre
		// e poi tutti gli altri ragionevoli...
		NSToolbarPrintItemIdentifier, NSToolbarShowFontsItemIdentifier,
		NSToolbarCustomizeToolbarItemIdentifier, NSToolbarFlexibleSpaceItemIdentifier, 
		NSToolbarSpaceItemIdentifier, NSToolbarSeparatorItemIdentifier, nil];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	// il pulsante di stampa non funziona; dopotutto, nessuno gli ha detto di stampare
// metodo delegato opzionale
// questo metodo e' chiamato quando si aggiunge un elemento alla toolbar
	// questo metodo e' il posto dove inserire codice per predisporre gli item
	// notif e' la toolbar, l'item lo ricavo usando la chiave @"item"
- (void) 
toolbarWillAddItem: (NSNotification *) notif 
{
	// ricavo l'item che mi interessa
	NSToolbarItem *addedItem = [[notif userInfo] objectForKey: @"item"];
	// se l'elemento e' l'icona per stampare...
	if ([[addedItem itemIdentifier] isEqual: NSToolbarPrintItemIdentifier]) 
	{
		// aggiungo il tooltip
		[addedItem setToolTip: @"Print the Catalog"];
		[addedItem setAction: @selector(print:)];
	}
}  
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo inviato al target per validare l'item
// questo metodo serve per abilitare/disabilitare item sulla toolbar
- (BOOL) 
validateToolbarItem: (NSToolbarItem *) toolbarItem 
{
	// l'item salva va bene solo se il documento e' stato cambiato
	if ([[toolbarItem itemIdentifier] isEqual: TBItId_SaveItem]) 
		return ( [[self document] isDocumentEdited] ) ;
	// tutti gli altri sono abilitati
	return ( YES );
}

// ----------------------------------------------------------------------------------------
// metodi per la gestione del drag'n'drop

- (unsigned int)
draggingEntered:(id<NSDraggingInfo>)sender
{
	NSPasteboard *pboard;

	// dico che accetto qualsiasi cosa che sia un path
	pboard = [sender draggingPasteboard];
	if ([[pboard types] indexOfObject:NSFilenamesPboardType] != NSNotFound) {
		return NSDragOperationLink;
	}
	return NSDragOperationNone;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (unsigned int)
draggingUpdated:(id<NSDraggingInfo>)sender
{
	return NSDragOperationLink;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
draggingExited:(id<NSDraggingInfo>)sender
{
	return ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (BOOL)
prepareForDragOperation:(id<NSDraggingInfo>)sender
{
	// questo e' il momento di decidere se fare il drop
	NSPasteboard *pboard;

	pboard = [sender draggingPasteboard];
	// in primo luogo, droppo solo path di file
	if ([[pboard types] indexOfObject:NSFilenamesPboardType] != NSNotFound) 
	{
		// recupero la lista degli elementi draggati
		NSArray		* pList = [ pboard propertyListForType: NSFilenamesPboardType ] ;
		// la metto in un enumerator
		NSEnumerator 	* enumerator = [pList objectEnumerator];
		// recupero anche la lista dei volumi correntemente mondati
		NSArray		* volumes = [ [NSWorkspace sharedWorkspace] mountedLocalVolumePaths ] ;
		NSString	* volPath ;
		// dico che accetto il drop solo di volumi
		while ((volPath = [enumerator nextObject])) 
		{
			// se cerco di droppare un elemento che non e' un volume
			if ( ! [ volumes containsObject: volPath ] )
				return ( NO );	// il drop non si fa
		}
		// se arrivo qui, tutti gli elementi droppati sono volumi,
		// posso eseguire l'operazione
		return ( YES );
	}
	// se arrivo qui, gli elementi droppati non sono nemmeno path...
	return ( NO );	
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (BOOL)
performDragOperation:(id<NSDraggingInfo>)sender
{
	NSPasteboard 	* dragPasteboard;
	NSArray		* pList ;

	// recupero la pasteboard
	dragPasteboard = [sender draggingPasteboard];
	// da qui, la lista degli elementi draggati
	pList = [ dragPasteboard propertyListForType: NSFilenamesPboardType ] ;
	[ [self document] performAddFilesModal: pList ];
	// ovviamente, il documento e' stato sporcato
	[ [self document] updateChangeCount: NSChangeDone ] ;
	return ( YES) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
concludeDragOperation:(id<NSDraggingInfo>)sender
{
	[ [self document] refreshWinsWithListSts: @"" coverSts: @"" ];
	return ;
}

// --------------------------------------------------------------------- 
// metodi comuni a tutti i controllori di finestra del documento
// aggiornamento del contenuto della finestra
- (void)
refreshWindow 
{
	[ fullList reloadData ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// impostazione della barra di stato
- (void) 
setStsStrings: (NSString *) sts1Val 
	andSts: (NSString *) st2Val;
{
	if ( sts1Val != nil )
		[ statusBar setStringValue: sts1Val];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// vista principale della finestra
- (NSOutlineView*) 
getMainView 
{
	return ( fullList );
}

@end

// --------------------------------------------------------------------- 
// aggiunge o elimina una colonna di una tablecolumn
// addRem = TRUE, aggiunge, FALSE elimina
void
setupColumn( NSOutlineView *outView, BOOL addRem, NSString * colId )
{
	NSTableColumn	*tmpTC ;

	// vedo se esiste una colonna relativa
	tmpTC = [ outView tableColumnWithIdentifier: colId] ;
	// se non c'e', e devo rimuoverla...
	if ( (tmpTC == nil) && (addRem == FALSE) )
		return ; 	// devo fare nulla
	// se c'e', e devo aggiungerla
	if ( tmpTC != nil && addRem )
		return ;	// devo fare nulla
	// se non c'e', e devo aggiungerla
	if ( tmpTC == nil && addRem )
	{
		// creo la NSTableColumn con l'identificatore
		tmpTC = [ [NSTableColumn alloc] initWithIdentifier: colId ];
		// imposto il titolo della colonna
		[[tmpTC headerCell] setStringValue: NSLocalizedString( colId, colId ) ];
		// modifico le caratteristiche visive della colonna
		configureColumn( tmpTC, colId );
		// aggiungo la colonna
		[outView addTableColumn: tmpTC ];		 
		return ;	// ho finito
	}
	// se arrivo qui, sicuramente c'e' la colonna e devo rimuoverla
	[outView removeTableColumn: tmpTC ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// modifica una tablecolumn aggiungendo formatter
void
configureColumn( NSTableColumn *tc, NSString * colId )
{
	// i formatter sono creati autorelease perche' passano in gestione alla tablecolumn
	if ( [ colId isEqual: COLID_OSTYPE ] )
	{
		TOS9TCForm 	*myDF1 = [[[ TOS9TCForm alloc ] init ] autorelease ];
		[[tc dataCell ] setFormatter:  myDF1 ];
		return ;
	}
	if ( [ colId isEqual: COLID_OSCREATOR ] )
	{
		TOS9TCForm 	*myDF1 = [[[ TOS9TCForm alloc ] init ] autorelease ];
		[[tc dataCell ] setFormatter:  myDF1 ];
		return ;
	}
	if ( [ colId isEqual: COLID_FILESIZE ] )
	{
		FileSizeForm 	*myDF2 = [[[ FileSizeForm alloc ] init ] autorelease ];
		[[tc dataCell ] setFormatter:  myDF2 ];
		// gia' che ci sono, sbandiero a destra
		[[tc dataCell   ] setAlignment: NSRightTextAlignment];
		[[tc headerCell ] setAlignment: NSRightTextAlignment];
		return ;
	}
	if ( [ colId isEqual: COLID_POSIXPERM ] )
	{
		FilePosixPerm 	*myDF3 = [[[ FilePosixPerm alloc ] init ] autorelease ];
		[[tc dataCell ] setFormatter:  myDF3 ];
		return ;
	}
	if ( [ colId isEqual: COLID_MODDATE ] )
	{
		NSDateFormatter *myDF4 = [[NSDateFormatter alloc]
			initWithDateFormat:@"%d %b %Y" allowNaturalLanguage:YES];
		[[tc dataCell ] setFormatter:  myDF4 ];
		return ;
	}
	if ( [ colId isEqual: COLID_CREATDATE ] )
	{
		NSDateFormatter *myDF4 = [[NSDateFormatter alloc]
			initWithDateFormat:@"%d %b %Y" allowNaturalLanguage:YES];
		[[tc dataCell ] setFormatter:  myDF4 ];
		return ;
	}
	if ( [ colId isEqual: COLID_ADD2PRINT ] )
	{
		NSButtonCell		* add2print = nil ;
		add2print = [[[ NSButtonCell alloc] init ] autorelease ];
		[ add2print setButtonType: NSSwitchButton ];
		[ add2print setTitle:@""] ;
		[ tc setWidth: 16 ];
		[ tc setResizable: FALSE ];
		[ [ tc headerCell] setStringValue: @"P?" ];
		[ tc setDataCell:add2print];
	}
}
